import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { SetConfigurationCommand } from './set-configuration.command';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import {
  ConfigurationIdentifier,
  ConfigurationType,
  SetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';
import { ConfigurationsManagerService } from '../../services/configurations-manager.service';
import { ArgumentInvalidException } from '@mobicoop/ddd-library';

@CommandHandler(SetConfigurationCommand)
export class SetConfigurationService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: SetConfigurationRepositoryPort,
    private readonly configurationsManager: ConfigurationsManagerService,
  ) {}

  async execute(
    command: SetConfigurationCommand,
  ): Promise<ConfigurationIdentifier> {
    const configurationType: ConfigurationType =
      this.configurationsManager.identifierType(
        command.configurationIdentifier,
      );
    const value: any = this.configurationsManager.cast(
      `${command.value}`,
      configurationType,
    );
    if (isNaN(value)) throw new ArgumentInvalidException('Bad value');
    return await this.configurationRepository.set(
      command.configurationIdentifier.domain,
      command.configurationIdentifier.key,
      value,
    );
  }
}
