import { IQueryHandler, QueryHandler } from '@nestjs/cqrs';
import { GetConfigurationQuery } from './get-configuration.query';
import { Inject } from '@nestjs/common';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import {
  ConfigurationValue,
  GetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';
import { ConfigurationsManagerService } from '../../services/configurations-manager.service';

@QueryHandler(GetConfigurationQuery)
export class GetConfigurationQueryHandler implements IQueryHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: GetConfigurationRepositoryPort,
    private readonly configurationsManager: ConfigurationsManagerService,
  ) {}
  async execute(query: GetConfigurationQuery): Promise<ConfigurationValue> {
    return await this.configurationRepository.get(
      query.configurationIdentifier.domain,
      query.configurationIdentifier.key,
      this.configurationsManager.identifierType(query.configurationIdentifier),
    );
  }
}
