import {
  GetConfigurationRepositoryPort,
  NotFoundException,
  SetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { Inject, Injectable, OnApplicationBootstrap } from '@nestjs/common';
import { ConfigurationsManagerService } from './configurations-manager.service';
import { Config } from '@src/config/config';

@Injectable()
export class PopulateService implements OnApplicationBootstrap {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly getConfigurationRepository: GetConfigurationRepositoryPort,
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly setConfigurationRepository: SetConfigurationRepositoryPort,
    private readonly configurationsManager: ConfigurationsManagerService,
  ) {}

  async onApplicationBootstrap() {
    await this._populate();
  }

  private _populate = async (): Promise<void> => {
    const configs: Config[] = this.configurationsManager.list();
    await Promise.all(
      configs.map((config: Config) => this._populateConfig(config)),
    );
  };

  private _populateConfig = async <T>(configuration: T): Promise<void> => {
    let key: keyof typeof configuration;
    const config: Config = configuration as Config;
    for (key in configuration) {
      try {
        if (key !== 'domain')
          await this.getConfigurationRepository.get(
            config.domain,
            key,
            this.configurationsManager.configurationType(configuration[key]),
          );
      } catch (error: any) {
        if (error instanceof NotFoundException) {
          this.setConfigurationRepository.set(
            config.domain,
            key,
            `${configuration[key]}`,
          );
        }
      }
    }
  };
}
