import {
  ConfigurationDomain,
  ConfigurationType,
} from '@mobicoop/configuration-module';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';
import { ConfigurationsManagerService } from '@modules/configuration/core/application/services/configurations-manager.service';
import { ConfigurationResponseDto } from '@modules/configuration/interface/dtos/configuration.response.dto';
import { Test } from '@nestjs/testing';

const mockConfigurationsManagerService = {
  configurationType: jest.fn().mockImplementation(() => ConfigurationType.INT),
};

describe('Configuration Mapper', () => {
  let configurationMapper: ConfigurationMapper;

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      providers: [
        ConfigurationMapper,
        {
          provide: ConfigurationsManagerService,
          useValue: mockConfigurationsManagerService,
        },
      ],
    }).compile();
    configurationMapper = module.get<ConfigurationMapper>(ConfigurationMapper);
  });

  it('should be defined', () => {
    expect(configurationMapper).toBeDefined();
  });

  it('should map configuration to response', async () => {
    const mapped: ConfigurationResponseDto = configurationMapper.toResponse(
      {
        domain: ConfigurationDomain.CARPOOL,
        key: 'seatsProposed',
      },
      '3',
    );
    expect(mapped.domain).toBe('CARPOOL');
    expect(mapped.key).toBe('seatsProposed');
    expect(mapped.value).toBe('3');
  });
});
