import { Test, TestingModule } from '@nestjs/testing';
import { GetConfigurationQueryHandler } from '@modules/configuration/core/application/queries/get-configuration/get-configuration.query-handler';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { GetConfigurationQuery } from '@modules/configuration/core/application/queries/get-configuration/get-configuration.query';
import {
  ConfigurationDomain,
  ConfigurationType,
  ConfigurationValue,
} from '@mobicoop/configuration-module';
import { ConfigurationsManagerService } from '@modules/configuration/core/application/services/configurations-manager.service';

const configurationValue: ConfigurationValue = '3';

const mockConfigurationRepository = {
  get: jest.fn().mockImplementation(() => configurationValue),
};

const mockConfigurationsManagerService = {
  identifierType: jest.fn().mockImplementation(() => ConfigurationType.INT),
};

describe('Get Configuration Query Handler', () => {
  let getConfigurationQueryHandler: GetConfigurationQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        {
          provide: ConfigurationsManagerService,
          useValue: mockConfigurationsManagerService,
        },
        GetConfigurationQueryHandler,
      ],
    }).compile();

    getConfigurationQueryHandler = module.get<GetConfigurationQueryHandler>(
      GetConfigurationQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(getConfigurationQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return a configuration value', async () => {
      const getConfigurationQuery = new GetConfigurationQuery(
        ConfigurationDomain.CARPOOL,
        'seatsProposed',
      );
      const configurationValue: ConfigurationValue =
        await getConfigurationQueryHandler.execute(getConfigurationQuery);
      expect(configurationValue).toBe('3');
    });
  });
});
